string formatTime(uint32 time)
{
    uint32 seconds = time / 1000 % 60;
    uint32 minutes = time / 1000 / 60 % 60;
    uint32 hours = time / 1000 / 60 / 60;

    string date = "";

    if (hours > 0) {
        date += formatNumber(hours) + "'";
    }

    date += formatNumber(minutes) + ':' + formatNumber(seconds) + '.' + formatThousandthsNumber(time % 1000);

    return date;
}

string formatNumber(uint32 number)
{
    if (number < 10) {
        return "0" + number;
    }

    return tostring(number);
}

string formatThousandthsNumber(uint32 number)
{
    if (number < 10) return "00" + number;
    if (number < 100) return "0" + number;

    return tostring(number);
}

int ParseStringTime(const string &in time) {
    string[] raceTimeSplit = time.Replace(').replay.gbx', '').Replace("''", "'").Split("'");

    if (raceTimeSplit[raceTimeSplit.Length - 1].Length == 1) raceTimeSplit[raceTimeSplit.Length - 1] = '0' + raceTimeSplit[raceTimeSplit.Length - 1] + '0';
    else raceTimeSplit[raceTimeSplit.Length - 1] += '0';

    int raceTime = Text::ParseInt(raceTimeSplit[raceTimeSplit.Length - 2] + raceTimeSplit[raceTimeSplit.Length - 1]);

    raceTime += Text::ParseInt(raceTimeSplit[raceTimeSplit.Length - 3]) * 60000;
    if (raceTimeSplit.Length > 3) raceTime += Text::ParseInt(raceTimeSplit[raceTimeSplit.Length - 4]) * (60000 * 60);
    return raceTime; 
}

string formatCleanerManiaplanet(const string &in name)
{
    return Regex::Replace(name, "\\$([0-9a-fA-F]{1,3}|[iIoOnNmMwWsSzZtTgG<>]|[lLhHpP](\\[[^\\]]+\\])?)", '');
}

string openplanetUIDisplay(const string &in name)
{
    string replacedName = Regex::Replace(name, "\\$([iIoOnNmMwWsSzZtTgG<>]|[lLhHpP](\\[[^\\]]+\\])?)", '');
    return Regex::Replace(replacedName, "\\$", '\\$');
}

string GetGhostName(int index) 
{
#if MP4
    if (GetDataMgr().Ghosts[index] is null || GetDataMgr().Ghosts[index].Result is null) return discoveredGhost.Nickname;
    else return GetDataMgr().Ghosts[index].Nickname;
#elif TURBO
    if (GetDataMgr().Ghosts[index] is null || GetDataMgr().Ghosts[index].RaceResult is null) return discoveredGhost.Nickname;
    else return GetDataMgr().Ghosts[index].Nickname;
#endif
}

int GetGhostTime(int index) 
{
#if MP4
    return GetApp().PlaygroundScript.DataFileMgr.Ghosts[index].Result.Time;
#elif TURBO
    return GetApp().PlaygroundScript.DataMgr.Ghosts[index].RaceResult.Time;
#endif
}

CGameManiaPlanetScriptAPI@ GetManiaPlanetAPI() {
    auto maniaplanet = cast<CGameManiaPlanet>(GetApp());
    return cast<CGameManiaPlanetScriptAPI@>(maniaplanet.ManiaPlanetScriptAPI);
}

#if MP4
    CGameDataFileManagerScript@ GetDataMgr() {return GetApp().PlaygroundScript.DataFileMgr;}
#elif TURBO
    CGameDataManagerScript@ GetDataMgr() {return GetApp().PlaygroundScript.DataMgr;}
#endif